/** @param {Creep} creep **/
export function runAsHarvester(creep: Creep) {
  if (creep.store.getFreeCapacity() > 0) {
    let sources = creep.room.find(FIND_SOURCES);
    if (creep.harvest(sources[0]) === ERR_NOT_IN_RANGE) {
      creep.moveTo(sources[0], { visualizePathStyle: { stroke: '#ffaa00' } });
    }
  } else {
    let targets = creep.room.find(FIND_STRUCTURES, {
      filter: (structure) => {
        return (
          (structure.structureType === STRUCTURE_EXTENSION ||
            structure.structureType === STRUCTURE_SPAWN ||
            structure.structureType === STRUCTURE_TOWER) &&
          structure.store.getFreeCapacity(RESOURCE_ENERGY) > 0
        );
      }
    });

    if (targets.length > 0) {
      if (creep.transfer(targets[0], RESOURCE_ENERGY) === ERR_NOT_IN_RANGE) {
        creep.moveTo(targets[0], { visualizePathStyle: { stroke: '#ffffff' } });
      }
    } else {
      const spawns = creep.room.find(FIND_MY_SPAWNS);
      if (spawns.length > 0) {
        creep.say('to spawn');
        creep.moveTo(spawns[0]);
      }
    }
  }
}

/** @param {Creep} creep **/
export function runAsBuilder(creep: Creep) {
  const memory = creep.memory as CreepMemory & { building: boolean | undefined };

  if (memory.building && creep.store[RESOURCE_ENERGY] === 0) {
    memory.building = false;
    creep.say('🔄 harvest');
  }

  if (!memory.building && creep.store.getFreeCapacity() === 0) {
    memory.building = true;
    creep.say('🚧 build');
  }

  if (memory.building) {
    const targets = creep.room.find(FIND_CONSTRUCTION_SITES);
    if (targets.length > 0) {
      if (creep.build(targets[0]) == ERR_NOT_IN_RANGE) {
        creep.moveTo(targets[0], { visualizePathStyle: { stroke: '#ffffff' } });
      }
    } else {
      const spawns = creep.room.find(FIND_MY_SPAWNS);
      if (spawns.length > 0) {
        creep.say('to spawn');
        creep.moveTo(spawns[0]);
      }
    }
  } else {
    const sources = creep.room.find(FIND_SOURCES);
    if (creep.harvest(sources[0]) == ERR_NOT_IN_RANGE) {
      creep.moveTo(sources[0], { visualizePathStyle: { stroke: '#ffaa00' } });
    }
  }
}

/** @param {Creep} creep **/
export function runAsUpgrader(creep: Creep) {
  const memory = creep.memory as CreepMemory & { upgrading: boolean | undefined };

  if (memory.upgrading && creep.store[RESOURCE_ENERGY] === 0) {
    memory.upgrading = false;
    creep.say('🔄 harvest');
  }

  if (!memory.upgrading && creep.store.getFreeCapacity() === 0) {
    memory.upgrading = true;
    creep.say('⚡ upgrade');
  }

  if (memory.upgrading) {
    if (creep.room.controller && creep.upgradeController(creep.room.controller) == ERR_NOT_IN_RANGE) {
      creep.moveTo(creep.room.controller, { visualizePathStyle: { stroke: '#ffffff' } });
    }
  } else {
    const sources = creep.room.find(FIND_SOURCES);
    if (creep.harvest(sources[0]) == ERR_NOT_IN_RANGE) {
      creep.moveTo(sources[0], { visualizePathStyle: { stroke: '#ffaa00' } });
    }
  }
}
