import { ErrorMapper } from 'utils/ErrorMapper';
import { runAsBuilder, runAsHarvester, runAsUpgrader } from './harvester';

const ALPHABET = 'abcdefghijklmnopqrstuvwxyz1234567890';
const ALPHABET_LENGTH = ALPHABET.length - 1;

function generateId(count: number): string {
  let str = '';
  for (let i = 0; i < count; ++i) {
    let symbolIndex = Math.floor(Math.random() * ALPHABET_LENGTH);
    str += ALPHABET[symbolIndex];
  }
  return str;
}

export function uniqId(prefix: string = 'id'): string {
  return prefix + generateId(16);
}

const ROLE_HARVESTER = 'harvester';
const ROLE_UPGRADER = 'upgrader';
const ROLE_BUILDER = 'builder';

// When compiling TS to JS and bundling with rollup, the line numbers and file names in error messages change
// This utility uses source maps to get the line numbers and file names of the original, TS source code
export const loop = ErrorMapper.wrapLoop(() => {
  console.log(`Current game tick is ${Game.time}`);

  // Automatically delete memory of missing creeps
  for (const name in Memory.creeps) {
    if (!(name in Game.creeps)) {
      delete Memory.creeps[name];
    }
  }

  // Create new creeps
  const HARVESTER_CREEP_COUNT = 1;
  const harvesterCreeps = Object.values(Game.creeps).filter((c) => c.memory.role === ROLE_HARVESTER);
  if (harvesterCreeps.length < HARVESTER_CREEP_COUNT) {
    const firstSpawn = _.first(Object.values(Game.spawns));
    const name = uniqId(ROLE_HARVESTER);
    const err = firstSpawn.spawnCreep([WORK, CARRY, MOVE], name, { memory: { role: ROLE_HARVESTER } as CreepMemory });
    console.log('Err', err);
  }

  const UPGRADER_CREEP_COUNT = 2;
  const upgraderCreeps = Object.values(Game.creeps).filter((c) => c.memory.role === ROLE_UPGRADER);
  if (upgraderCreeps.length < UPGRADER_CREEP_COUNT) {
    const firstSpawn = _.first(Object.values(Game.spawns));
    const name = uniqId(ROLE_UPGRADER);
    const err = firstSpawn.spawnCreep([WORK, CARRY, MOVE], name, { memory: { role: ROLE_UPGRADER } as CreepMemory });
    console.log('Err', err);
  }

  const BUILDER_CREEP_COUNT = 2;
  const builderCreeps = Object.values(Game.creeps).filter((c) => c.memory.role === ROLE_BUILDER);
  if (builderCreeps.length < BUILDER_CREEP_COUNT) {
    const firstSpawn = _.first(Object.values(Game.spawns));
    const name = uniqId(ROLE_BUILDER);
    const err = firstSpawn.spawnCreep([WORK, CARRY, MOVE], name, { memory: { role: ROLE_BUILDER } as CreepMemory });
    console.log('Err', err);
  }

  // Process current creeps
  for (let name in Game.creeps) {
    const creep = Game.creeps[name];
    if (creep.memory.role === ROLE_HARVESTER) {
      runAsHarvester(creep);
    }
    if (creep.memory.role === ROLE_UPGRADER) {
      runAsUpgrader(creep);
    }
    if (creep.memory.role === ROLE_BUILDER) {
      runAsBuilder(creep);
    }
  }
});
